# -*- coding: utf-8 -*-
# This file is part of the Rainbow Project
"""
Rainbow is a micro-framework for Python RPC+P&S communication over WebSockets.
"""

__author__ = 'Jesús Arroyo Torrens'
__email__ = 'jesus.arroyo@bq.com'
__copyright__ = 'Copyright (c) 2015 Mundo Reader S.L.'
__license__ = 'GPLv2'

__version__ = '0.0.4'

try:
    from rainbow.dealer import Dealer
    from rainbow.broker import Broker

    dealer = Dealer()
    broker = Broker()
except:
    pass


def register(key):
    def decorator(function):
        dealer.register(key, function)
        return function
    return decorator


def publish(event=None, data=None):
    broker.publish(event, data)


def run(host='0.0.0.0', webserver=False, webbrowser=False, debug=False):
    print 'Running server ' + host
    if webserver:
        from rainbow import webserver
        webserver.run(host)
        if webbrowser:
            import webbrowser
            webbrowser.open('http://' + host + ':8000')
    broker.run(host)
    dealer.run_forever(host, debug)



import json
import unittest
from rainbow import register
from rainbow.dealer import _call_manager as dealer


class RainbowTest(unittest.TestCase):

    def setUp(self):
        # Register functions

        @register('pi')
        def pi():
            return 3.141592

        @register('hello')
        def hello(name):
            return 'Hello, ' + name

        @register('sum')
        def sum(a, b):
            return a + b

        @register('list')
        def _list(a, b, c, d):
            return [a, b, c, d]

        @register('dict')
        def _dict(key, value):
            ret = {}
            ret[key] = value
            return ret

    def test_call_pi(self):
        ret = json.loads('3.141592')
        self.assertEqual(dealer.call(u'pi'), ret)

    def test_call_hello(self):
        args = json.loads('["world"]')
        ret = json.loads('"Hello, world"')
        self.assertEqual(dealer.call(u'hello', args=args), ret)

    def test_call_hello_kw(self):
        kwargs = json.loads('{"name":"world"}')
        ret = json.loads('"Hello, world"')
        self.assertEqual(dealer.call(u'hello', kwargs=kwargs), ret)

    def test_call_sum(self):
        kwargs = json.loads('{"a":3,"b":5}')
        ret = json.loads('8')
        self.assertEqual(dealer.call(u'sum', kwargs=kwargs), ret)

    def test_call_sum_kw(self):
        args = json.loads('[3]')
        kwargs = json.loads('{"b":5}')
        ret = json.loads('8')
        self.assertEqual(dealer.call(u'sum', args=args, kwargs=kwargs), ret)

    def test_call_sum_akw(self):
        args = json.loads('[3,5]')
        ret = json.loads('8')
        self.assertEqual(dealer.call(u'sum', args=args), ret)

    def test_call_list(self):
        kwargs = json.loads('{"b":null,"a":3,"c":"code","d":0.5}')
        ret = json.loads('[3, null,"code",0.5]')
        self.assertEqual(dealer.call(u'list', kwargs=kwargs), ret)

    def test_call_list_kw(self):
        args = json.loads('[3,null]')
        kwargs = json.loads('{"d":0.5,"c":"code"}')
        ret = json.loads('[3,null,"code",0.5]')
        self.assertEqual(dealer.call(u'list', args=args, kwargs=kwargs), ret)

    def test_call_dict(self):
        kwargs = json.loads('{"key":"a","value":34}')
        ret = json.loads('{"a":34}')
        self.assertEqual(dealer.call(u'dict', kwargs=kwargs), ret)

if __name__ == '__main__':  # pragma: no cover
    unittest.main()



# -*- coding: utf-8 -*-
# This file is part of the Rainbow Project

__author__ = 'Jesús Arroyo Torrens'
__email__ = 'jesus.arroyo@bq.com'
__copyright__ = 'Copyright (c) 2015 Mundo Reader S.L.'
__license__ = 'GPLv2'

import json
from collections import OrderedDict
from geventwebsocket import WebSocketApplication, WebSocketServer, Resource

from rainbow.singleton import Singleton


class JSONRPCException(Exception):
    def __init__(self, code, message, data=None):
        self.code = code
        self.message = message
        self.data = data


class ParseError(JSONRPCException):
    def __init__(self):
        JSONRPCException.__init__(self, -32700, 'Parse error')


class InvalidRequest(JSONRPCException):
    def __init__(self):
        JSONRPCException.__init__(self, -32600, 'Invalid Request')


class MethodNotFound(JSONRPCException):
    def __init__(self):
        JSONRPCException.__init__(self, -32601, 'Method not found')


class InvalidParams(JSONRPCException):
    def __init__(self):
        JSONRPCException.__init__(self, -32602, 'Invalid params')


class InternalError(JSONRPCException):
    def __init__(self, data):
        JSONRPCException.__init__(self, -32603, 'Internal error', data)


@Singleton
class CallManager(object):
    def __init__(self):
        self.functions = {}

    def register(self, key, function):
        """
        Register function in dictionary with the provided key
        """
        self.functions[key] = function

    def process_request(self, request):
        """
        Process request according to JSON-RPC 2.0 Specs:
            <http://www.jsonrpc.org/specification>
        Input and output data types are JSON string
        """
        response = None
        try:
            request = json.loads(request)
        except:
            # Parse error
            response = {'jsonrpc': '2.0',
                        'error': {'code': ParseError().code,
                                  'message': ParseError().message},
                        'id': None}
        else:
            if isinstance(request, list):
                if not request:
                    # Empty array
                    response = {'jsonrpc': '2.0',
                                'error': {'code': InvalidRequest().code,
                                          'message': InvalidRequest().message},
                                'id': None}
                else:
                    # Batch call
                    response = []
                    for req in request:
                        res = self._single_request(req)
                        if res is not None:
                            # Not notification
                            response += [res]
                    if not response:
                        # All notifications
                        response = None
            else:
                # Single call
                response = self._single_request(request)
        finally:
            if response is not None:
                return json.dumps(response)

    def _single_request(self, request):
        response = None
        try:
            self._verify_request(request)
            result = self._execute_request(request)
        except JSONRPCException as e:
            response = {'jsonrpc': '2.0',
                        'error': {'code': e.code, 'message': e.message},
                        'id': None}
            if isinstance(e, InternalError):
                response['error']['data'] = e.data
            if isinstance(e, MethodNotFound) or isinstance(e, InvalidParams):
                response['id'] = request['id']
        else:
            if 'id' in request:
                response = {'jsonrpc': '2.0',
                            'result': result,
                            'id': request['id']}
            else:
                # Notification
                response = None
        finally:
            return response

    def _verify_request(self, request):
        if not isinstance(request, dict) or \
           'jsonrpc' not in request or \
           request['jsonrpc'] != '2.0':
            raise InvalidRequest

    def _execute_request(self, request):
        args = {}
        kwargs = {}
        result = None
        if 'params' in request and isinstance(request['params'], list):
            args = request['params']
        if 'params' in request and isinstance(request['params'], dict):
            kwargs = request['params']
        result = self.call(request['method'], args, kwargs)
        return result

    def call(self, key, args={}, kwargs={}):
        result = None
        if isinstance(key, unicode):
            if key == '_functions':
                return self.functions.keys()
            elif key in self.functions:
                try:
                    result = self.functions[key](*args, **kwargs)
                except TypeError as e:
                    raise InvalidParams
                except Exception as e:
                    raise InternalError(e.message)
                else:
                    return result
            else:
                raise MethodNotFound
        else:
            raise InvalidRequest


_call_manager = CallManager()


class DealerApplication(WebSocketApplication):
    def on_message(self, message):
        response = _call_manager.process_request(message)
        if response is not None:
            self.ws.send(response)


@Singleton
class Dealer(object):
    """
    It is responsible for routing a call originating from the Caller
    to the Callee and route back results or errors
    """
    def register(self, key, function):
        _call_manager.register(key, function)
    def run_forever(self, host='0.0.0.0', debug=False):
        try:
            dealer_server = WebSocketServer(
                (host, 8080),
                Resource(OrderedDict({'/': DealerApplication})),
                debug=debug)
            dealer_server.serve_forever()
        except KeyboardInterrupt:
            dealer_server.close()



import json
import unittest
from rainbow import register
from rainbow.dealer import _call_manager as dealer


def jsonok(text):
    return json.dumps(json.loads(text))


class RainbowTest(unittest.TestCase):

    def setUp(self):
        # Register functions

        @register('subtract')
        def subtract(minuend, subtrahend):
            return minuend - subtrahend

        @register('update')
        def update(a, b, c, d, e):
            pass

        @register('foobar')
        def foobar():
            pass

        @register('sum')
        def sum(a, b, c):
            return a + b + c

        @register('notify_sum')
        def notify_sum(a, b, c):
            pass

        @register('notify_hello')
        def notify_hello(a):
            pass

        @register('get_data')
        def get_data():
            return ["hello", 5]

    def test_subtract_pos_params(self):
        request = '{"jsonrpc": "2.0","method": "subtract","params": [42, 23],"id": 1}'
        response = '{"jsonrpc": "2.0", "result": 19, "id": 1}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_subtract_named_params(self):
        request = '{"jsonrpc": "2.0", "method": "subtract", ' \
                  '"params": {"subtrahend": 23, "minuend": 42}, "id": 3}'
        response = '{"jsonrpc": "2.0", "result": 19, "id": 3}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_invalid_request_subtract(self):
        request = '{"method": "subtract","params": [42, 23],"id": 1}'
        response = '{"jsonrpc": "2.0", "error": {"code": -32600, ' \
                   '"message": "Invalid Request"}, "id": null}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_invalid_param_subtract(self):
        request = '{"jsonrpc": "2.0","method": "subtract","params": [42],"id": "id"}'
        response = '{"jsonrpc": "2.0", "error": {"code": -32602, ' \
                   '"message": "Invalid params"}, "id": "id"}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_invalid_params_subtract(self):
        request = '{"jsonrpc": "2.0","method": "subtract","params": [42,"2"],"id": 1}'
        response = '{"jsonrpc": "2.0", "error": {"code": -32602, ' \
                   '"message": "Invalid params"}, "id": 1}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_foobar(self):
        request = '{"jsonrpc": "2.0", "method": "foobar", "id": 1}'
        response = '{"jsonrpc": "2.0", "result": null, "id": 1}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_notification_update(self):
        request = '{"jsonrpc": "2.0", "method": "update", "params": [1,2,3,4,5]}'
        response = None
        self.assertEqual(dealer.process_request(request), response)

    def test_notification_foobar(self):
        request = '{"jsonrpc": "2.0", "method": "foobar"}'
        response = None
        self.assertEqual(dealer.process_request(request), response)

    def test_non_existent_method(self):
        request = '{"jsonrpc": "2.0", "method": "bar", "id": "1"}'
        response = '{"jsonrpc": "2.0", "error": {"code": -32601, ' \
                   '"message": "Method not found"}, "id": "1"}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_invalid_json(self):
        request = '{"jsonrpc": "2.0", "method": "foobar, "params": "bar", "baz]'
        response = '{"jsonrpc": "2.0", "error": {"code": -32700, ' \
                   '"message": "Parse error"}, "id": null}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_invalid_request(self):
        request = '{"jsonrpc": "2.0", "method": 1, "params": "bar"}'
        response = '{"jsonrpc": "2.0", "error": {"code": -32600, ' \
                   '"message": "Invalid Request"}, "id": null}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_batch_invalid_json(self):
        request = '[{"jsonrpc": "2.0", "method": "sum", "params": [1,2,4], "id": "1"},' \
                  '{"jsonrpc": "2.0", "method"]'
        response = '{"jsonrpc": "2.0", "error": {"code": -32700, ' \
                   '"message": "Parse error"}, "id": null}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_batch_emtpy_array(self):
        request = '[]'
        response = '{"jsonrpc": "2.0", "error": {"code": -32600, ' \
                   '"message": "Invalid Request"}, "id": null}'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_batch_invalid_not_empty(self):
        request = '[1]'
        response = '[{"jsonrpc": "2.0", "error": {"code": -32600, ' \
                   '"message": "Invalid Request"}, "id": null}]'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_batch_invalid(self):
        request = '[1,2,3]'
        response = '[' \
            '{"jsonrpc": "2.0", "error": {"code": -32600, ' \
            '"message": "Invalid Request"}, "id": null},' \
            '{"jsonrpc": "2.0", "error": {"code": -32600, ' \
            '"message": "Invalid Request"}, "id": null},' \
            '{"jsonrpc": "2.0", "error": {"code": -32600, ' \
            '"message": "Invalid Request"}, "id": null}]'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_call_batch(self):
        request = '[' \
            '{"jsonrpc": "2.0", "method": "sum", "params": [1,2,4], "id": "1"},' \
            '{"jsonrpc": "2.0", "method": "notify_hello", "params": [7]},' \
            '{"jsonrpc": "2.0", "method": "subtract", "params": [42,23], "id": "2"},' \
            '{"foo": "boo"},' \
            '{"jsonrpc": "2.0", "method": "foo.get", "params": {"name": "myself"}, "id": "5"},' \
            '{"jsonrpc": "2.0", "method": "get_data", "id": "9"}]'
        response = '[' \
            '{"jsonrpc": "2.0", "result": 7, "id": "1"},' \
            '{"jsonrpc": "2.0", "result": 19, "id": "2"},' \
            '{"jsonrpc": "2.0", "error":{"code": -32600, "message": "Invalid Request"}, ' \
            '"id": null},' \
            '{"jsonrpc": "2.0", "error":{"code": -32601, "message": "Method not found"}, ' \
            '"id": "5"},' \
            '{"jsonrpc": "2.0", "result": ["hello", 5], "id": "9"}]'
        self.assertEqual(dealer.process_request(request), jsonok(response))

    def test_call_batch_notification(self):
        request = '[{"jsonrpc": "2.0", "method": "notify_sum", "params": [1,2,4]},' \
                  '{"jsonrpc": "2.0", "method": "notify_hello", "params": [7]}]'
        response = None
        self.assertEqual(dealer.process_request(request), response)

if __name__ == '__main__':  # pragma: no cover
    unittest.main()



# -*- coding: utf-8 -*-
# This file is part of the Rainbow Project

__author__ = 'Jesús Arroyo Torrens'
__email__ = 'jesus.arroyo@bq.com'
__copyright__ = 'Copyright (c) 2015 Mundo Reader S.L.'
__license__ = 'GPLv2'


def Singleton(cls):
    class cls_w(cls):
        _instance = None

        def __new__(cls, *args, **kwargs):
            if cls_w._instance is None:
                cls_w._instance = super(cls_w, cls).__new__(cls, *args, **kwargs)
                cls_w._instance.__initialized = False
            return cls_w._instance

        def __init__(cls, *args, **kwargs):
            if not cls_w._instance.__initialized:
                super(cls_w, cls).__init__(*args, **kwargs)
                cls_w._instance.__initialized = True

    cls_w.__name__ = cls.__name__
    return cls_w



#!/usr/bin/env python

from setuptools import setup

import rainbow

setup(name='pyrainbow',
      version=rainbow.__version__,
      description=rainbow.__doc__,
      author=rainbow.__author__,
      author_email=rainbow.__email__,
      url='https://github.com/bqlabs/rainbow',
      download_url='https://pypi.python.org/pypi/pyrainbow',
      license=rainbow.__license__,
      packages=['rainbow'],
      package_data={'rainbow': ['index.html']},
      install_requires=['paste>=2.0.2',
                        'gevent>=1.0.2',
                        'gevent-zeromq>=0.2.5',
                        'gevent-websocket>=0.9.5'],
      extras_require=dict(develop=['nose>=1.3.1']),
      classifiers=['Development Status :: 3 - Alpha',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
                   'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries',
                   'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
                   'Topic :: Internet :: WWW/HTTP :: WSGI',
                   'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
                   'Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware',
                   'Topic :: Internet :: WWW/HTTP :: WSGI :: Server',
                   'Topic :: Software Development :: Libraries :: Application Frameworks',
                   'Programming Language :: Python :: 2.7']
      )



# -*- coding: utf-8 -*-
# This file is part of the Rainbow Project

__author__ = 'Jesús Arroyo Torrens'
__email__ = 'jesus.arroyo@bq.com'
__copyright__ = 'Copyright (c) 2015 Mundo Reader S.L.'
__license__ = 'GPLv2'

import json
import gevent
import datetime
from gevent_zeromq import zmq
from geventwebsocket.handler import WebSocketHandler

from rainbow.singleton import Singleton


@Singleton
class Broker(object):
    """
    It is responsible for sending the event information
    to the subscribers when a topic is published
    """
    def run(self, host='0.0.0.0'):
        context = zmq.Context()
        gevent.spawn(zmq_server, context)
        gevent.pywsgi.WSGIServer((host, 8081), BrokerWebSocket(context),
                                 handler_class=WebSocketHandler).start()
        # Event socket
        self.socket = context.socket(zmq.PUB)
        self.socket.connect('tcp://' + host + ':5000')

    def publish(self, event=None, data=None):
        topic = {'time': datetime.datetime.now().isoformat(),
                 'event': event,
                 'data': data}
        self.socket.send(json.dumps(topic))


def zmq_server(context):
    """
    Funnel messages coming from the external tcp socket to an inproc socket
    """
    sock_incoming = context.socket(zmq.SUB)
    sock_outgoing = context.socket(zmq.PUB)
    sock_incoming.bind('tcp://*:5000')
    sock_outgoing.bind('inproc://queue')
    sock_incoming.setsockopt(zmq.SUBSCRIBE, "")
    while True:
        msg = sock_incoming.recv()
        sock_outgoing.send(msg)


class BrokerWebSocket(object):
    """
    Funnel messages coming from an inproc zmq socket to the websocket
    """
    def __init__(self, context):
        self.context = context

    def __call__(self, environ, start_response):
        ws = environ['wsgi.websocket']
        sock = self.context.socket(zmq.SUB)
        sock.setsockopt(zmq.SUBSCRIBE, "")
        sock.connect('inproc://queue')
        while True:
            msg = sock.recv()
            ws.send(msg)



# -*- coding: utf-8 -*-
# This file is part of the Rainbow Project

__author__ = 'Jesús Arroyo Torrens'
__email__ = 'jesus.arroyo@bq.com'
__copyright__ = 'Copyright (c) 2015 Mundo Reader S.L.'
__license__ = 'GPLv2'


def run(host='0.0.0.0'):
    import os
    import gevent.pywsgi
    import paste.urlparser
    http_server = gevent.pywsgi.WSGIServer(
        (host, 8000), paste.urlparser.StaticURLParser(os.path.dirname(__file__)))
    http_server.start()
